# -*- coding: utf-8 -*-
#
# Univention S4 Connector
#  this file defines the mapping beetween S4 and UCS
#
# Copyright 2004-2017 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import univention.s4connector.s4
import univention.s4connector.s4.mapping
import univention.s4connector.s4.password
import univention.s4connector.s4.sid_mapping
import univention.s4connector.s4.ntsecurity_descriptor
import univention.s4connector.s4.dns
import univention.s4connector.s4.dc
import univention.s4connector.s4.computer

global_ignore_subtree=['cn=univention,dc=deadlock42,dc=intranet', 'cn=policies,dc=deadlock42,dc=intranet', 'cn=shares,dc=deadlock42,dc=intranet', 'cn=printers,dc=deadlock42,dc=intranet', 'cn=networks,dc=deadlock42,dc=intranet', 'cn=kerberos,dc=deadlock42,dc=intranet', 'cn=dhcp,dc=deadlock42,dc=intranet', 'cn=mail,dc=deadlock42,dc=intranet', 'cn=nagios,dc=deadlock42,dc=intranet', 'CN=RAS and IAS Servers Access Check,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=FileLinks,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=WinsockServices,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=RID Manager$,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=Dfs-Configuration,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=Server,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=ComPartitionSets,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=ComPartitions,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=IP Security,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=DFSR-GlobalSettings,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=DomainUpdates,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=Password Settings Container,CN=System,DC=DEADLOCK42,DC=INTRANET', 'DC=RootDNSServers,CN=MicrosoftDNS,CN=System,DC=DEADLOCK42,DC=INTRANET', 'DC=RootDNSServers,CN=MicrosoftDNS,DC=DomainDnsZones,DC=DEADLOCK42,DC=INTRANET', 'DC=RootDNSServers,CN=MicrosoftDNS,DC=ForestDnsZones,DC=DEADLOCK42,DC=INTRANET', 'DC=..TrustAnchors,CN=MicrosoftDNS,DC=ForestDnsZones,DC=DEADLOCK42,DC=INTRANET', 'CN=File Replication Service,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=RpcServices,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=Meetings,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=AdminSDHolder,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=BCKUPKEY_c490e871-a375-4b76-bd24-711e9e49fe5e Secret,CN=System,DC=DEADLOCK42,DC=INTRANET', 'CN=BCKUPKEY_PREFERRED Secret,CN=System,DC=DEADLOCK42,DC=INTRANET', 'ou=Grp Policy Users,DC=DEADLOCK42,DC=INTRANET', 'cn=ForeignSecurityPrincipals,DC=DEADLOCK42,DC=INTRANET', 'cn=Program Data,DC=DEADLOCK42,DC=INTRANET', 'cn=Configuration,DC=DEADLOCK42,DC=INTRANET', 'cn=opsi,dc=deadlock42,dc=intranet', 'cn=Microsoft Exchange System Objects,DC=DEADLOCK42,DC=INTRANET', 'CN=WMIPolicy,CN=System,DC=DEADLOCK42,DC=INTRANET']


s4_mapping = {
	'user': univention.s4connector.property (
			ucs_default_dn='cn=users,dc=deadlock42,dc=intranet',
			con_default_dn='cn=users,DC=DEADLOCK42,DC=INTRANET',

			ucs_module='users/user',

			# read, write, sync, none
			sync_mode='sync',

			scope='sub',

			con_search_filter='(&(objectClass=user)(!(objectClass=computer))(userAccountControl:1.2.840.113556.1.4.803:=512))',
			match_filter='(&(|(&(objectClass=posixAccount)(objectClass=krb5Principal))(objectClass=user))(!(objectClass=univentionHost)))',
			ignore_filter='(|(uid=root)(CN=root)(uid=ucs-s4sync)(CN=ucs-s4sync))',

			ignore_subtree = global_ignore_subtree,
			
			con_create_objectclass=['top', 'user', 'person', 'organizationalPerson'],

			dn_mapping_function=[ univention.s4connector.s4.user_dn_mapping ],

			# aus UCS Modul
			attributes= {
					'samAccountName': univention.s4connector.attribute (
							ucs_attribute='username',
							ldap_attribute='uid',
							con_attribute='sAMAccountName',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'givenName' : univention.s4connector.attribute (
							ucs_attribute='firstname',
							ldap_attribute='givenName',
							con_attribute='givenName',
							single_value=True,
						),
					'displayName': univention.s4connector.attribute (
							ucs_attribute='displayName',
							ldap_attribute='displayName',
							con_attribute='displayName',
							single_value=True,
						),
					'sn': univention.s4connector.attribute (
							ucs_attribute='lastname',
							ldap_attribute='sn',
							con_attribute='sn',
							single_value=True,
						),
					
					'sid': univention.s4connector.attribute (
						sync_mode='read',
						mapping=(None, univention.s4connector.s4.sid_mapping.sid_to_ucs_mapping),
						ldap_attribute='sambaSID',
						ucs_attribute='sambaRID',
						con_attribute='objectSid',
						single_value=True,
						compare_function=univention.s4connector.s4.compare_sid_lists,
					), 

			},

			# These functions can extend the addlist while
			# creating an object in S4. Parameters are
			#	s4connector, property_type, object, addlist, serverctrls
			con_create_extenstions = [
							univention.s4connector.s4.add_primary_group_to_addlist,
			],
			ucs_create_functions = [
							univention.s4connector.set_ucs_passwd_user,
						 	univention.s4connector.check_ucs_lastname_user,
						 	univention.s4connector.set_primary_group_user,
							
						 	],

			post_con_create_functions = [
							univention.s4connector.s4.normalise_userAccountControl,
						 	],

			post_con_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_ucs_to_s4,
							univention.s4connector.s4.primary_group_sync_from_ucs,
							univention.s4connector.s4.object_memberships_sync_from_ucs,
							univention.s4connector.s4.disable_user_from_ucs,
							],

			post_ucs_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_s4_to_ucs,
							univention.s4connector.s4.primary_group_sync_to_ucs,
							univention.s4connector.s4.object_memberships_sync_to_ucs,
							univention.s4connector.s4.disable_user_to_ucs,
							],

			post_attributes={
					'organisation': univention.s4connector.attribute (
							ucs_attribute='organisation',
							ldap_attribute='o',
							con_attribute='company',
						),
					'organisation': univention.s4connector.attribute (
							ucs_attribute='organisation',
							ldap_attribute='o',
							con_attribute='company',
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),
					'mailPrimaryAddress': univention.s4connector.attribute (
							ucs_attribute='mailPrimaryAddress',
							ldap_attribute='mailPrimaryAddress',
							con_attribute='mail',
							reverse_attribute_check = True,
							single_value=True,
						),
					'street': univention.s4connector.attribute (
							ucs_attribute='street',
							ldap_attribute='street',
							con_attribute='streetAddress',
							single_value=True,
						),
					'city': univention.s4connector.attribute (
							ucs_attribute='city',
							ldap_attribute='l',
							con_attribute='l',
							single_value=True,
						),
					'postcode': univention.s4connector.attribute (
							ucs_attribute='postcode',
							ldap_attribute='postalCode',
							con_attribute='postalCode',
							single_value=True,
						),
					'sambaWorkstations': univention.s4connector.attribute (
							ucs_attribute='sambaUserWorkstations',
							ldap_attribute='sambaUserWorkstations',
							con_attribute='userWorkstations',
							single_value=True,
						),
					#'sambaLogonHours': univention.s4connector.attribute (
					#		ucs_attribute='sambaLogonHours',
					#		ldap_attribute='sambaLogonHours',
					#		con_attribute='logonHours',
					#	),
					'profilepath': univention.s4connector.attribute (
							ucs_attribute='profilepath',
							ldap_attribute='sambaProfilePath',
							con_attribute='profilePath',
							single_value=True,
						),
					'scriptpath': univention.s4connector.attribute (
							ucs_attribute='scriptpath',
							ldap_attribute='sambaLogonScript',
							con_attribute='scriptPath',
							single_value=True,
						),
					'homeDrive': univention.s4connector.attribute (
							ucs_attribute='homedrive',
							ldap_attribute='sambaHomeDrive',
							con_attribute='homeDrive',
							single_value=True,
						),
					'homeDirectory': univention.s4connector.attribute (
							ucs_attribute='sambahome',
							ldap_attribute='sambaHomePath',
							con_attribute='homeDirectory',
							reverse_attribute_check = True,
							single_value=True,
						),
					'telephoneNumber': univention.s4connector.attribute (
							ucs_attribute='phone',
							ldap_attribute='telephoneNumber',
							con_attribute='telephoneNumber',
							con_other_attribute='otherTelephone',
						),
					'homePhone': univention.s4connector.attribute (
							ucs_attribute='homeTelephoneNumber',
							ldap_attribute='homePhone',
							con_attribute='homePhone',
							con_other_attribute='otherHomePhone',
						),
					'mobilePhone': univention.s4connector.attribute (
							ucs_attribute='mobileTelephoneNumber',
							ldap_attribute='mobile',
							con_attribute='mobile',
							con_other_attribute='otherMobile',
						),
					'pager': univention.s4connector.attribute (
							ucs_attribute='pagerTelephoneNumber',
							ldap_attribute='pager',
							con_attribute='pager',
							con_other_attribute='otherPager',
						),
			},

		),

	'group': univention.s4connector.property (
			ucs_default_dn='cn=groups,dc=deadlock42,dc=intranet',
			con_default_dn='cn=Users,DC=DEADLOCK42,DC=INTRANET',

			ucs_module='groups/group',

			sync_mode='sync',

			scope='sub',

			ignore_filter='(|(cn=Windows Hosts)(cn=Authenticated Users)(cn=World Authority)(cn=Everyone)(cn=Null Authority)(cn=Nobody)(cn=Enterprise Domain Controllers)(cn=Remote Interactive Logon)(cn=SChannel Authentication)(cn=Digest Authentication)(cn=Terminal Server User)(cn=NTLM Authentication)(cn=Other Organization)(cn=This Organization)(cn=Anonymous Logon)(cn=Network Service)(cn=Creator Group)(cn=Creator Owner)(cn=Local Service)(cn=Owner Rights)(cn=Interactive)(cn=Restricted)(cn=Network)(cn=Service)(cn=Dialup)(cn=System)(cn=Batch)(cn=Proxy)(cn=IUSR)(cn=Self))',


			ignore_subtree = global_ignore_subtree,
			
			con_search_filter='objectClass=group',

			con_create_objectclass=['top', 'group'],

			# These functions can extend the addlist while
			# creating an object in S4. Parameters are
			#	s4connector, property_type, object, addlist, serverctrls
			con_create_extenstions = [
							univention.s4connector.s4.check_for_local_group_and_extend_serverctrls_and_sid,
			],

			post_con_modify_functions=[
							
							univention.s4connector.s4.group_members_sync_from_ucs,
							univention.s4connector.s4.object_memberships_sync_from_ucs
							],

			post_ucs_modify_functions=[
							
							univention.s4connector.s4.group_members_sync_to_ucs,
							univention.s4connector.s4.object_memberships_sync_to_ucs
							],

			dn_mapping_function=[ univention.s4connector.s4.group_dn_mapping ],

			attributes= {
					'cn': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='cn',
							con_attribute='sAMAccountName',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),
					'groupType': univention.s4connector.attribute (
							ucs_attribute='adGroupType',
							ldap_attribute='univentionGroupType',
							con_attribute='groupType',
							single_value=True,
					),

					'mailAddress': univention.s4connector.attribute (
							ucs_attribute='mailAddress',
							ldap_attribute='mailPrimaryAddress',
							con_attribute='mail',
							reverse_attribute_check = True,
							single_value=True,
					),
					
					'sid': univention.s4connector.attribute (
						sync_mode='read',
						mapping=(None, univention.s4connector.s4.sid_mapping.sid_to_ucs_mapping),
						ldap_attribute='sambaSID',
						ucs_attribute='sambaRID',
						con_attribute='objectSid',
						single_value=True,
						compare_function=univention.s4connector.s4.compare_sid_lists,
					), 

				},

			mapping_table = {

			'cn': [
				(u'Printer-Admins', u'Print Operators'),
				]

			},

		),
	'dc': univention.s4connector.property (
			ucs_default_dn='cn=dc,cn=computers,dc=deadlock42,dc=intranet',
			con_default_dn='OU=Domain Controllers,DC=DEADLOCK42,DC=INTRANET',
			ucs_module='computers/windows_domaincontroller',
			ucs_module_others=['computers/domaincontroller_master', 'computers/domaincontroller_backup', 'computers/domaincontroller_slave'],
			con_search_filter='(&(objectClass=computer)(userAccountControl:1.2.840.113556.1.4.803:=532480))',

			position_mapping = [( ',cn=dc,cn=computers,dc=deadlock42,dc=intranet', ',ou=Domain Controllers,DC=DEADLOCK42,DC=INTRANET' )],

			match_filter='(|(&(objectClass=univentionDomainController)(univentionService=Samba 4))(objectClass=computer)(univentionServerRole=windows_domaincontroller))',

			dn_mapping_function=[ univention.s4connector.s4.dc_dn_mapping ],

			# Whether a DC joins to the samba 4 domain
			# the DC will be deleted.
			disable_delete_in_ucs = True,

			# Whether a DC is removed in UCS, the DC should be removed
			# in S4. By default a DC has a subobject wihtout any mapping
			# and this subobject would avoid a deletion of this DC in S4
			con_subtree_delete_objects = [ 'cn=rid set' ],

	
			sync_mode='sync',


			con_create_objectclass=['top', 'computer' ],
			
			con_create_attributes=[
									('userAccountControl', ['532480']),
								  ],
	
			post_con_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_ucs_to_s4,
							],
	
			post_ucs_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_s4_to_ucs_no_userpassword,
							univention.s4connector.s4.computer.checkAndConvertToMacOSX,
							],
	
			attributes= {
					'cn': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='cn',
							con_attribute='cn',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'samAccountName': univention.s4connector.attribute (
							ldap_attribute='uid',
							con_attribute='sAMAccountName',
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),
					'operatingSystem': univention.s4connector.attribute (
							ucs_attribute='operatingSystem',
							ldap_attribute='univentionOperatingSystem',
							con_attribute='operatingSystem',
							single_value=True,
						),
					'operatingSystemVersion': univention.s4connector.attribute (
							ucs_attribute='operatingSystemVersion',
							ldap_attribute='univentionOperatingSystemVersion',
							con_attribute='operatingSystemVersion',
							single_value=True,
						),
					
					'sid': univention.s4connector.attribute (
						sync_mode='read',
						mapping=(None, univention.s4connector.s4.sid_mapping.sid_to_ucs_mapping),
						ldap_attribute='sambaSID',
						ucs_attribute='sambaRID',
						con_attribute='objectSid',
						single_value=True,
						compare_function=univention.s4connector.s4.compare_sid_lists,
					), 

				},
	
		),
	'windowscomputer': univention.s4connector.property (
			ucs_default_dn='cn=computers,dc=deadlock42,dc=intranet',
			con_default_dn='cn=computers,DC=DEADLOCK42,DC=INTRANET',
			ucs_module='computers/windows',
			ucs_module_others=['computers/memberserver', 'computers/ucc', 'computers/linux', 'computers/ubuntu', 'computers/macos' ],

			sync_mode='sync',


			scope='sub',

			dn_mapping_function=[ univention.s4connector.s4.windowscomputer_dn_mapping ],

			con_search_filter='(&(objectClass=computer)(userAccountControl:1.2.840.113556.1.4.803:=4096))',

			# ignore_filter='userAccountControl=4096',
			match_filter='(|(&(objectClass=univentionWindows)(!(univentionServerRole=windows_domaincontroller)))(objectClass=computer)(objectClass=univentionMemberServer)(objectClass=univentionUbuntuClient)(objectClass=univentionLinuxClient)(objectClass=univentionMacOSClient)(objectClass=univentionCorporateClient))',

			ignore_subtree = global_ignore_subtree,


			con_create_objectclass=['top', 'computer' ],

			con_create_attributes=[('userAccountControl', ['4096'])],

			#post_con_create_functions = [ univention.connector.s4.computers.
			post_con_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_ucs_to_s4,
							],

			post_ucs_modify_functions=[
							
							univention.s4connector.s4.password.password_sync_s4_to_ucs_no_userpassword,
							univention.s4connector.s4.computer.checkAndConvertToMacOSX,
							univention.s4connector.s4.computer.windowscomputer_sync_s4_to_ucs_check_rename,
							],

			attributes= {
					'cn': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='cn',
							con_attribute='cn',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'samAccountName': univention.s4connector.attribute (
							ldap_attribute='uid',
							con_attribute='sAMAccountName',
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),
					'operatingSystem': univention.s4connector.attribute (
							ucs_attribute='operatingSystem',
							ldap_attribute='univentionOperatingSystem',
							con_attribute='operatingSystem',
							single_value=True,
						),
					'operatingSystemVersion': univention.s4connector.attribute (
							ucs_attribute='operatingSystemVersion',
							ldap_attribute='univentionOperatingSystemVersion',
							con_attribute='operatingSystemVersion',
							single_value=True,
						),
					
					'sid': univention.s4connector.attribute (
						sync_mode='read',
						mapping=(None, univention.s4connector.s4.sid_mapping.sid_to_ucs_mapping),
						ldap_attribute='sambaSID',
						ucs_attribute='sambaRID',
						con_attribute='objectSid',
						single_value=True,
						compare_function=univention.s4connector.s4.compare_sid_lists,
					), 

				},

		),

	'dns': univention.s4connector.property (
			ucs_default_dn='cn=dns,dc=deadlock42,dc=intranet',
			con_default_dn='CN=MicrosoftDNS,DC=DomainDnsZones,DC=DEADLOCK42,DC=INTRANET',
			ucs_module='dns/dns',
			
			identify=univention.s4connector.s4.dns.identify,
			sync_mode='sync',

			scope='sub',

			con_search_filter='(|(objectClass=dnsNode)(objectClass=dnsZone))',

			dn_mapping_function=[ univention.s4connector.s4.dns.dns_dn_mapping ],

			ignore_filter='(|(dc=_ldap._tcp.Default-First-Site-Name._site))',
			ignore_subtree = global_ignore_subtree,
			
			con_sync_function = univention.s4connector.s4.dns.ucs2con,
			ucs_sync_function = univention.s4connector.s4.dns.con2ucs,
		),

	'msGPO': univention.s4connector.property (
			ucs_module='container/msgpo',

			sync_mode='sync',

			scope='sub',

			con_search_filter='(&(objectClass=container)(objectClass=groupPolicyContainer))',

			ignore_filter='',

			ignore_subtree = global_ignore_subtree,
			
			con_create_objectclass=['top', 'container', 'groupPolicyContainer' ],

			attributes= {
					'cn': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='cn',
							con_attribute='cn',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),
					'displayName': univention.s4connector.attribute (
							ucs_attribute='displayName',
							ldap_attribute='displayName',
							con_attribute='displayName',
							single_value=True,
						),
					'msGPOFlags': univention.s4connector.attribute (
							ucs_attribute='msGPOFlags',
							ldap_attribute='msGPOFlags',
							con_attribute='flags',
							single_value=True,
						),
					'msGPOVersionNumber': univention.s4connector.attribute (
							ucs_attribute='msGPOVersionNumber',
							ldap_attribute='msGPOVersionNumber',
							con_attribute='versionNumber',
							single_value=True,
						),
					'msGPOSystemFlags': univention.s4connector.attribute (
							ucs_attribute='msGPOSystemFlags',
							ldap_attribute='msGPOSystemFlags',
							con_attribute='systemFlags',
							single_value=True,
						),
					'msGPOFunctionalityVersion': univention.s4connector.attribute (
							ucs_attribute='msGPOFunctionalityVersion',
							ldap_attribute='msGPOFunctionalityVersion',
							con_attribute='gPCFunctionalityVersion',
							single_value=True,
						),
					'msGPOFileSysPath': univention.s4connector.attribute (
							ucs_attribute='msGPOFileSysPath',
							ldap_attribute='msGPOFileSysPath',
							con_attribute='gPCFileSysPath',
							single_value=True,
						),
					'msGPOMachineExtensionNames': univention.s4connector.attribute (
							ucs_attribute='msGPOMachineExtensionNames',
							ldap_attribute='msGPOMachineExtensionNames',
							con_attribute='gPCMachineExtensionNames',
							single_value=True,
						),
					'msGPOUserExtensionNames': univention.s4connector.attribute (
							ucs_attribute='msGPOUserExtensionNames',
							ldap_attribute='msGPOUserExtensionNames',
							con_attribute='gPCUserExtensionNames',
							single_value=True,
						),
					'msGPOWQLFilter': univention.s4connector.attribute (
							ucs_attribute='msGPOWQLFilter',
							ldap_attribute='msGPOWQLFilter',
							con_attribute='gPCWQLFilter',
							single_value=True,
						),
				},

		),


	'container': univention.s4connector.property (
			ucs_module='container/cn',

			sync_mode='sync',


			scope='sub',

			con_search_filter='(&(|(objectClass=container)(objectClass=builtinDomain))(!(objectClass=groupPolicyContainer)))', # builtinDomain is cn=builtin (with group cn=Administrators)

			ignore_filter='(|(cn=mail)(cn=kerberos)(cn=MicrosoftDNS))',


			ignore_subtree = global_ignore_subtree,
			
			con_create_objectclass=['top', 'container' ],

			attributes= {
					'cn': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='cn',
							con_attribute='cn',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),

					'gPLink': univention.s4connector.attribute (
							ucs_attribute='gPLink',
							ldap_attribute='msGPOLink',
							con_attribute='gPLink',
							single_value=True,
						),
	

				},

		),

	'ou': univention.s4connector.property (
			ucs_module='container/ou',

			sync_mode='sync',


			scope='sub',

			con_search_filter='objectClass=organizationalUnit',



			ignore_subtree = global_ignore_subtree,

			con_create_objectclass=[ 'top', 'organizationalUnit' ],

			attributes= {
					'ou': univention.s4connector.attribute (
							ucs_attribute='name',
							ldap_attribute='ou',
							con_attribute='ou',
							required=1,
							compare_function=univention.s4connector.compare_lowercase,
							single_value=True,
						),
					'description': univention.s4connector.attribute (
							ucs_attribute='description',
							ldap_attribute='description',
							con_attribute='description',
							single_value=True,
						),

					'gPLink': univention.s4connector.attribute (
							ucs_attribute='gPLink',
							ldap_attribute='msGPOLink',
							con_attribute='gPLink',
							single_value=True,
						),
	

				},
		),
	'container_dc': univention.s4connector.property (
			ucs_module='container/dc',
			ucs_default_dn='cn=samba,dc=deadlock42,dc=intranet',
			con_default_dn='DC=DEADLOCK42,DC=INTRANET',
			
			sync_mode='sync',


			scope='sub',

			identify=univention.s4connector.s4.dc.identify,

			con_search_filter='(|(&(objectClass=domain)(!(|(name=DomainDnsZones)(name=ForestDnsZones))))(objectClass=sambaDomainName))',



			ignore_subtree = global_ignore_subtree,
			
			con_sync_function = univention.s4connector.s4.dc.ucs2con,
			ucs_sync_function = univention.s4connector.s4.dc.con2ucs,

		),
}



